@@ -16,24 +16,29 @@ use hpke::{
     aead::{Aead, AeadCtxR, AeadCtxS, AeadTag, AesGcm128, AesGcm256, ChaCha20Poly1305},
     kdf::{HkdfSha256, HkdfSha384, HkdfSha512, Kdf as KdfTrait},
     kem::{DhP256HkdfSha256, Kem as KemTrait, X25519HkdfSha256},
-    kex::{Deserializable, KeyExchange, Serializable},
-    op_mode::PskBundle,
-    setup_receiver, setup_sender, EncappedKey, HpkeError, OpModeR, OpModeS,
+    setup_receiver, setup_sender, Deserializable, HpkeError, OpModeR, OpModeS, PskBundle,
+    Serializable,
 };
 
 use rand::{rngs::StdRng, CryptoRng, RngCore, SeedableRng};
 
 trait AgileAeadCtxS {
-    fn seal(&mut self, plaintext: &mut [u8], aad: &[u8]) -> Result<AgileAeadTag, HpkeError>;
+    fn seal_in_place_detached(
+        &mut self,
+        plaintext: &mut [u8],
+        aad: &[u8],
+    ) -> Result<AgileAeadTag, AgileHpkeError>;
+    fn seal(&mut self, plaintext: &[u8], aad: &[u8]) -> Result<Vec<u8>, AgileHpkeError>;
 }
 
 trait AgileAeadCtxR {
-    fn open(
+    fn open_in_place_detached(
         &mut self,
         ciphertext: &mut [u8],
         aad: &[u8],
         tag_bytes: &[u8],
     ) -> Result<(), AgileHpkeError>;
+    fn open(&mut self, ciphertext: &[u8], aad: &[u8]) -> Result<Vec<u8>, AgileHpkeError>;
 }
 
 type AgileAeadTag = Vec<u8>;
@@ -58,20 +63,33 @@ impl From<HpkeError> for AgileHpkeError {
 }
 
 impl<A: Aead, Kdf: KdfTrait, Kem: KemTrait> AgileAeadCtxS for AeadCtxS<A, Kdf, Kem> {
-    fn seal(&mut self, plaintext: &mut [u8], aad: &[u8]) -> Result<Vec<u8>, HpkeError> {
-        self.seal(plaintext, aad).map(|tag| tag.to_bytes().to_vec())
+    fn seal_in_place_detached(
+        &mut self,
+        plaintext: &mut [u8],
+        aad: &[u8],
+    ) -> Result<Vec<u8>, AgileHpkeError> {
+        self.seal_in_place_detached(plaintext, aad)
+            .map(|tag| tag.to_bytes().to_vec())
+            .map_err(Into::into)
+    }
+    fn seal(&mut self, plaintext: &[u8], aad: &[u8]) -> Result<Vec<u8>, AgileHpkeError> {
+        self.seal(plaintext, aad).map_err(Into::into)
     }
 }
 
 impl<A: Aead, Kdf: KdfTrait, Kem: KemTrait> AgileAeadCtxR for AeadCtxR<A, Kdf, Kem> {
-    fn open(
+    fn open_in_place_detached(
         &mut self,
         ciphertext: &mut [u8],
         aad: &[u8],
         tag_bytes: &[u8],
     ) -> Result<(), AgileHpkeError> {
         let tag = AeadTag::<A>::from_bytes(tag_bytes)?;
-        self.open(ciphertext, aad, &tag).map_err(|e| e.into())
+        self.open_in_place_detached(ciphertext, aad, &tag)
+            .map_err(Into::into)
+    }
+    fn open(&mut self, ciphertext: &[u8], aad: &[u8]) -> Result<Vec<u8>, AgileHpkeError> {
+        self.open(ciphertext, aad).map_err(Into::into)
     }
 }
 
@@ -102,7 +120,7 @@ impl AeadAlg {
         Ok(res)
     }
 
-    fn to_u16(&self) -> u16 {
+    fn to_u16(self) -> u16 {
         match self {
             AeadAlg::AesGcm128 => 0x01,
             AeadAlg::AesGcm256 => 0x02,
@@ -138,7 +156,7 @@ impl KdfAlg {
         Ok(res)
     }
 
-    fn to_u16(&self) -> u16 {
+    fn to_u16(self) -> u16 {
         match self {
             KdfAlg::HkdfSha256 => 0x01,
             KdfAlg::HkdfSha384 => 0x02,
@@ -155,37 +173,6 @@ impl KdfAlg {
     }
 }
 
-#[derive(Clone, Copy, Debug, Eq, PartialEq)]
-enum KexAlg {
-    X25519,
-    X448,
-    DhP256,
-    DhP384,
-    DhP521,
-}
-
-impl KexAlg {
-    fn name(&self) -> &'static str {
-        match self {
-            KexAlg::X25519 => "X25519",
-            KexAlg::X448 => "X448",
-            KexAlg::DhP256 => "P256",
-            KexAlg::DhP384 => "P384",
-            KexAlg::DhP521 => "P521",
-        }
-    }
-
-    fn get_pubkey_len(&self) -> usize {
-        match self {
-            KexAlg::X25519 => 32,
-            KexAlg::X448 => 56,
-            KexAlg::DhP256 => 65,
-            KexAlg::DhP384 => 97,
-            KexAlg::DhP521 => 133,
-        }
-    }
-}
-
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 enum KemAlg {
     X25519HkdfSha256,
@@ -219,7 +206,7 @@ impl KemAlg {
         Ok(res)
     }
 
-    fn to_u16(&self) -> u16 {
+    fn to_u16(self) -> u16 {
         match self {
             KemAlg::DhP256HkdfSha256 => 0x10,
             KemAlg::DhP384HkdfSha384 => 0x11,
@@ -229,16 +216,6 @@ impl KemAlg {
         }
     }
 
-    fn kex_alg(&self) -> KexAlg {
-        match self {
-            KemAlg::X25519HkdfSha256 => KexAlg::X25519,
-            KemAlg::X448HkdfSha512 => KexAlg::X448,
-            KemAlg::DhP256HkdfSha256 => KexAlg::DhP256,
-            KemAlg::DhP384HkdfSha384 => KexAlg::DhP384,
-            KemAlg::DhP521HkdfSha512 => KexAlg::DhP521,
-        }
-    }
-
     fn kdf_alg(&self) -> KdfAlg {
         match self {
             KemAlg::X25519HkdfSha256 => KdfAlg::HkdfSha256,
@@ -252,37 +229,37 @@ impl KemAlg {
 
 #[derive(Clone)]
 struct AgilePublicKey {
-    kex_alg: KexAlg,
+    kem_alg: KemAlg,
     pubkey_bytes: Vec<u8>,
 }
 
 impl AgilePublicKey {
-    fn try_lift<Kex: KeyExchange>(&self) -> Result<Kex::PublicKey, AgileHpkeError> {
-        Kex::PublicKey::from_bytes(&self.pubkey_bytes).map_err(|e| e.into())
+    fn try_lift<Kem: KemTrait>(&self) -> Result<Kem::PublicKey, AgileHpkeError> {
+        Kem::PublicKey::from_bytes(&self.pubkey_bytes).map_err(|e| e.into())
     }
 }
 
 #[derive(Clone)]
 struct AgileEncappedKey {
-    kex_alg: KexAlg,
+    kem_alg: KemAlg,
     encapped_key_bytes: Vec<u8>,
 }
 
 impl AgileEncappedKey {
-    fn try_lift<Kex: KeyExchange>(&self) -> Result<EncappedKey<Kex>, AgileHpkeError> {
-        EncappedKey::<Kex>::from_bytes(&self.encapped_key_bytes).map_err(|e| e.into())
+    fn try_lift<Kem: KemTrait>(&self) -> Result<Kem::EncappedKey, AgileHpkeError> {
+        Kem::EncappedKey::from_bytes(&self.encapped_key_bytes).map_err(|e| e.into())
     }
 }
 
 #[derive(Clone)]
 struct AgilePrivateKey {
-    kex_alg: KexAlg,
+    kem_alg: KemAlg,
     privkey_bytes: Vec<u8>,
 }
 
 impl AgilePrivateKey {
-    fn try_lift<Kex: KeyExchange>(&self) -> Result<Kex::PrivateKey, AgileHpkeError> {
-        Kex::PrivateKey::from_bytes(&self.privkey_bytes).map_err(|e| e.into())
+    fn try_lift<Kem: KemTrait>(&self) -> Result<Kem::PrivateKey, AgileHpkeError> {
+        Kem::PrivateKey::from_bytes(&self.privkey_bytes).map_err(|e| e.into())
     }
 }
 
@@ -290,17 +267,15 @@ impl AgilePrivateKey {
 struct AgileKeypair(AgilePrivateKey, AgilePublicKey);
 
 impl AgileKeypair {
-    fn try_lift<Kex: KeyExchange>(
-        &self,
-    ) -> Result<(Kex::PrivateKey, Kex::PublicKey), AgileHpkeError> {
-        Ok((self.0.try_lift::<Kex>()?, self.1.try_lift::<Kex>()?))
+    fn try_lift<Kem: KemTrait>(&self) -> Result<(Kem::PrivateKey, Kem::PublicKey), AgileHpkeError> {
+        Ok((self.0.try_lift::<Kem>()?, self.1.try_lift::<Kem>()?))
     }
 
     fn validate(&self) -> Result<(), AgileHpkeError> {
-        if self.0.kex_alg != self.1.kex_alg {
+        if self.0.kem_alg != self.1.kem_alg {
             Err(AgileHpkeError::AlgMismatch(
-                (self.0.kex_alg.name(), "AgileKeypair::privkey"),
-                (self.1.kex_alg.name(), "AgileKeypair::pubkey"),
+                (self.0.kem_alg.name(), "AgileKeypair::privkey"),
+                (self.1.kem_alg.name(), "AgileKeypair::pubkey"),
             ))
         } else {
             Ok(())
@@ -310,18 +285,18 @@ impl AgileKeypair {
 
 // The leg work of agile_gen_keypair
 macro_rules! do_gen_keypair {
-    ($kem_ty:ty, $kex_alg:ident, $csprng:ident) => {{
+    ($kem_ty:ty, $kem_alg:ident, $csprng:ident) => {{
         type Kem = $kem_ty;
-        let kex_alg = $kex_alg;
+        let kem_alg = $kem_alg;
         let csprng = $csprng;
 
         let (sk, pk) = Kem::gen_keypair(csprng);
         let sk = AgilePrivateKey {
-            kex_alg: kex_alg,
+            kem_alg,
             privkey_bytes: sk.to_bytes().to_vec(),
         };
         let pk = AgilePublicKey {
-            kex_alg: kex_alg,
+            kem_alg,
             pubkey_bytes: pk.to_bytes().to_vec(),
         };
 
@@ -330,28 +305,27 @@ macro_rules! do_gen_keypair {
 }
 
 fn agile_gen_keypair<R: CryptoRng + RngCore>(kem_alg: KemAlg, csprng: &mut R) -> AgileKeypair {
-    let kex_alg = kem_alg.kex_alg();
     match kem_alg {
-        KemAlg::X25519HkdfSha256 => do_gen_keypair!(X25519HkdfSha256, kex_alg, csprng),
-        KemAlg::DhP256HkdfSha256 => do_gen_keypair!(DhP256HkdfSha256, kex_alg, csprng),
+        KemAlg::X25519HkdfSha256 => do_gen_keypair!(X25519HkdfSha256, kem_alg, csprng),
+        KemAlg::DhP256HkdfSha256 => do_gen_keypair!(DhP256HkdfSha256, kem_alg, csprng),
         _ => unimplemented!(),
     }
 }
 
 #[derive(Clone)]
 struct AgileOpModeR<'a> {
-    kex_alg: KexAlg,
+    kem_alg: KemAlg,
     op_mode_ty: AgileOpModeRTy<'a>,
 }
 
 impl<'a> AgileOpModeR<'a> {
-    fn try_lift<Kex: KeyExchange, Kdf: KdfTrait>(self) -> Result<OpModeR<'a, Kex>, AgileHpkeError> {
+    fn try_lift<Kem: KemTrait, Kdf: KdfTrait>(self) -> Result<OpModeR<'a, Kem>, AgileHpkeError> {
         let res = match self.op_mode_ty {
             AgileOpModeRTy::Base => OpModeR::Base,
             AgileOpModeRTy::Psk(bundle) => OpModeR::Psk(bundle.try_lift::<Kdf>()?),
-            AgileOpModeRTy::Auth(pk) => OpModeR::Auth(pk.try_lift::<Kex>()?),
+            AgileOpModeRTy::Auth(pk) => OpModeR::Auth(pk.try_lift::<Kem>()?),
             AgileOpModeRTy::AuthPsk(pk, bundle) => {
-                OpModeR::AuthPsk(pk.try_lift::<Kex>()?, bundle.try_lift::<Kdf>()?)
+                OpModeR::AuthPsk(pk.try_lift::<Kem>()?, bundle.try_lift::<Kdf>()?)
             }
         };
 
@@ -361,23 +335,23 @@ impl<'a> AgileOpModeR<'a> {
     fn validate(&self) -> Result<(), AgileHpkeError> {
         match &self.op_mode_ty {
             AgileOpModeRTy::Auth(pk) => {
-                if pk.kex_alg != self.kex_alg {
+                if pk.kem_alg != self.kem_alg {
                     return Err(AgileHpkeError::AlgMismatch(
-                        (self.kex_alg.name(), "AgileOpModeR::kex_alg"),
+                        (self.kem_alg.name(), "AgileOpModeR::kem_alg"),
                         (
-                            pk.kex_alg.name(),
-                            "AgileOpModeR::op_mode_ty::AgilePublicKey::kex_alg",
+                            pk.kem_alg.name(),
+                            "AgileOpModeR::op_mode_ty::AgilePublicKey::kem_alg",
                         ),
                     ));
                 }
             }
             AgileOpModeRTy::AuthPsk(pk, _) => {
-                if pk.kex_alg != self.kex_alg {
+                if pk.kem_alg != self.kem_alg {
                     return Err(AgileHpkeError::AlgMismatch(
-                        (self.kex_alg.name(), "AgileOpModeR::kex_alg"),
+                        (self.kem_alg.name(), "AgileOpModeR::kem_alg"),
                         (
-                            pk.kex_alg.name(),
-                            "AgileOpModeR::op_mode_ty::AgilePublicKey::kex_alg",
+                            pk.kem_alg.name(),
+                            "AgileOpModeR::op_mode_ty::AgilePublicKey::kem_alg",
                         ),
                     ));
                 }
@@ -399,18 +373,18 @@ enum AgileOpModeRTy<'a> {
 
 #[derive(Clone)]
 struct AgileOpModeS<'a> {
-    kex_alg: KexAlg,
+    kem_alg: KemAlg,
     op_mode_ty: AgileOpModeSTy<'a>,
 }
 
 impl<'a> AgileOpModeS<'a> {
-    fn try_lift<Kex: KeyExchange, Kdf: KdfTrait>(self) -> Result<OpModeS<'a, Kex>, AgileHpkeError> {
+    fn try_lift<Kem: KemTrait, Kdf: KdfTrait>(self) -> Result<OpModeS<'a, Kem>, AgileHpkeError> {
         let res = match self.op_mode_ty {
             AgileOpModeSTy::Base => OpModeS::Base,
             AgileOpModeSTy::Psk(bundle) => OpModeS::Psk(bundle.try_lift::<Kdf>()?),
-            AgileOpModeSTy::Auth(keypair) => OpModeS::Auth(keypair.try_lift::<Kex>()?),
+            AgileOpModeSTy::Auth(keypair) => OpModeS::Auth(keypair.try_lift::<Kem>()?),
             AgileOpModeSTy::AuthPsk(keypair, bundle) => {
-                OpModeS::AuthPsk(keypair.try_lift::<Kex>()?, bundle.try_lift::<Kdf>()?)
+                OpModeS::AuthPsk(keypair.try_lift::<Kem>()?, bundle.try_lift::<Kdf>()?)
             }
         };
 
@@ -421,24 +395,24 @@ impl<'a> AgileOpModeS<'a> {
         match &self.op_mode_ty {
             AgileOpModeSTy::Auth(keypair) => {
                 keypair.validate()?;
-                if keypair.0.kex_alg != self.kex_alg {
+                if keypair.0.kem_alg != self.kem_alg {
                     return Err(AgileHpkeError::AlgMismatch(
-                        (self.kex_alg.name(), "AgileOpModeS::kex_alg"),
+                        (self.kem_alg.name(), "AgileOpModeS::kem_alg"),
                         (
-                            keypair.0.kex_alg.name(),
-                            "AgileOpModeS::op_mode_ty::AgilePrivateKey::kex_alg",
+                            keypair.0.kem_alg.name(),
+                            "AgileOpModeS::op_mode_ty::AgilePrivateKey::kem_alg",
                         ),
                     ));
                 }
             }
             AgileOpModeSTy::AuthPsk(keypair, _) => {
                 keypair.validate()?;
-                if keypair.0.kex_alg != self.kex_alg {
+                if keypair.0.kem_alg != self.kem_alg {
                     return Err(AgileHpkeError::AlgMismatch(
-                        (self.kex_alg.name(), "AgileOpModeS::kex_alg"),
+                        (self.kem_alg.name(), "AgileOpModeS::kem_alg"),
                         (
-                            keypair.0.kex_alg.name(),
-                            "AgileOpModeS::op_mode_ty::AgilePrivateKey::kex_alg",
+                            keypair.0.kem_alg.name(),
+                            "AgileOpModeS::op_mode_ty::AgilePrivateKey::kem_alg",
                         ),
                     ));
                 }
@@ -549,13 +523,13 @@ where
     Kem: 'static + KemTrait,
     R: CryptoRng + RngCore,
 {
-    let kex_alg = mode.kex_alg;
-    let mode = mode.clone().try_lift::<Kem::Kex, Kdf>()?;
-    let pk_recip = pk_recip.try_lift::<Kem::Kex>()?;
+    let kem_alg = mode.kem_alg;
+    let mode = mode.clone().try_lift::<Kem, Kdf>()?;
+    let pk_recip = pk_recip.try_lift::<Kem>()?;
 
     let (encapped_key, aead_ctx) = setup_sender::<A, Kdf, Kem, _>(&mode, &pk_recip, info, csprng)?;
     let encapped_key = AgileEncappedKey {
-        kex_alg,
+        kem_alg,
         encapped_key_bytes: encapped_key.to_bytes().to_vec(),
     };
 
@@ -573,22 +547,16 @@ fn agile_setup_sender<R: CryptoRng + RngCore>(
 ) -> Result<(AgileEncappedKey, Box<dyn AgileAeadCtxS>), AgileHpkeError> {
     // Do all the necessary validation
     mode.validate()?;
-    if mode.kex_alg != pk_recip.kex_alg {
+    if mode.kem_alg != pk_recip.kem_alg {
         return Err(AgileHpkeError::AlgMismatch(
-            (mode.kex_alg.name(), "mode::kex_alg"),
-            (pk_recip.kex_alg.name(), "pk_recip::kex_alg"),
+            (mode.kem_alg.name(), "mode::kem_alg"),
+            (pk_recip.kem_alg.name(), "pk_recip::kem_alg"),
         ));
     }
-    if kem_alg.kex_alg() != mode.kex_alg {
+    if kem_alg != mode.kem_alg {
         return Err(AgileHpkeError::AlgMismatch(
-            (kem_alg.kex_alg().name(), "kem_alg::kex_alg"),
-            (mode.kex_alg.name(), "mode::kex_alg"),
-        ));
-    }
-    if pk_recip.kex_alg != mode.kex_alg {
-        return Err(AgileHpkeError::AlgMismatch(
-            (pk_recip.kex_alg.name(), "pk_recip::kex_alg"),
-            (mode.kex_alg.name(), "mode::kex_alg"),
+            (kem_alg.name(), "kem_alg::kem_alg"),
+            (mode.kem_alg.name(), "mode::kem_alg"),
         ));
     }
 
@@ -632,9 +600,9 @@ where
     Kdf: 'static + KdfTrait,
     Kem: 'static + KemTrait,
 {
-    let mode = mode.clone().try_lift::<Kem::Kex, Kdf>()?;
-    let (sk_recip, _) = recip_keypair.try_lift::<Kem::Kex>()?;
-    let encapped_key = encapped_key.try_lift::<Kem::Kex>()?;
+    let mode = mode.clone().try_lift::<Kem, Kdf>()?;
+    let (sk_recip, _) = recip_keypair.try_lift::<Kem>()?;
+    let encapped_key = encapped_key.try_lift::<Kem>()?;
 
     let aead_ctx = setup_receiver::<A, Kdf, Kem>(&mode, &sk_recip, &encapped_key, info)?;
     Ok(Box::new(aead_ctx))
@@ -652,22 +620,22 @@ fn agile_setup_receiver(
     // Do all the necessary validation
     recip_keypair.validate()?;
     mode.validate()?;
-    if mode.kex_alg != recip_keypair.0.kex_alg {
+    if mode.kem_alg != recip_keypair.0.kem_alg {
         return Err(AgileHpkeError::AlgMismatch(
-            (mode.kex_alg.name(), "mode::kex_alg"),
-            (recip_keypair.0.kex_alg.name(), "recip_keypair::kex_alg"),
+            (mode.kem_alg.name(), "mode::kem_alg"),
+            (recip_keypair.0.kem_alg.name(), "recip_keypair::kem_alg"),
         ));
     }
-    if kem_alg.kex_alg() != mode.kex_alg {
+    if kem_alg != mode.kem_alg {
         return Err(AgileHpkeError::AlgMismatch(
-            (kem_alg.kex_alg().name(), "kem_alg::kex_alg"),
-            (mode.kex_alg.name(), "mode::kex_alg"),
+            (kem_alg.name(), "kem_alg::kem_alg"),
+            (mode.kem_alg.name(), "mode::kem_alg"),
         ));
     }
-    if recip_keypair.0.kex_alg != encapped_key.kex_alg {
+    if recip_keypair.0.kem_alg != encapped_key.kem_alg {
         return Err(AgileHpkeError::AlgMismatch(
-            (recip_keypair.0.kex_alg.name(), "recip_keypair::kex_alg"),
-            (encapped_key.kex_alg.name(), "encapped_key::kex_alg"),
+            (recip_keypair.0.kem_alg.name(), "recip_keypair::kem_alg"),
+            (encapped_key.kem_alg.name(), "encapped_key::kem_alg"),
         ));
     }
 
@@ -718,7 +686,6 @@ fn main() {
         for &kem_alg in supported_kem_algs {
             for &kdf_alg in supported_kdf_algs {
                 let info = b"we're gonna agile him in his clavicle";
-                let kex_alg = kem_alg.kex_alg();
 
                 // Make a random sender keypair and PSK bundle
                 let sender_keypair = agile_gen_keypair(kem_alg, &mut csprng);
@@ -734,15 +701,14 @@ fn main() {
 
                 // Make two agreeing OpModes (AuthPsk is the most complicated, so we're just using
                 // that).
-                let op_mode_s_ty =
-                    AgileOpModeSTy::AuthPsk(sender_keypair.clone(), psk_bundle.clone());
+                let op_mode_s_ty = AgileOpModeSTy::AuthPsk(sender_keypair.clone(), psk_bundle);
                 let op_mode_s = AgileOpModeS {
-                    kex_alg,
+                    kem_alg,
                     op_mode_ty: op_mode_s_ty,
                 };
-                let op_mode_r_ty = AgileOpModeRTy::AuthPsk(sender_keypair.1, psk_bundle.clone());
+                let op_mode_r_ty = AgileOpModeRTy::AuthPsk(sender_keypair.1, psk_bundle);
                 let op_mode_r = AgileOpModeR {
-                    kex_alg,
+                    kem_alg,
                     op_mode_ty: op_mode_r_ty,
                 };
 
@@ -774,10 +740,8 @@ fn main() {
                 // Test an encryption-decryption round trip
                 let msg = b"paper boy paper boy";
                 let aad = b"all about that paper, boy";
-                let mut plaintext = *msg;
-                let tag = aead_ctx1.seal(&mut plaintext, aad).unwrap();
-                let mut ciphertext = plaintext;
-                aead_ctx2.open(&mut ciphertext, aad, &tag).unwrap();
+                let ciphertext = aead_ctx1.seal(msg, aad).unwrap();
+                aead_ctx2.open(&ciphertext, aad).unwrap();
                 let roundtrip_plaintext = ciphertext;
 
                 // Assert that the derived plaintext equals the original message
