@@ -1,8 +1,6 @@
-use crate::kem::Kem as KemTrait;
-
 use byteorder::{BigEndian, ByteOrder};
 use digest::{BlockInput, Digest, FixedOutput, Reset, Update};
-use generic_array::GenericArray;
+use generic_array::{typenum::Unsigned, GenericArray};
 use sha2::{Sha256, Sha384, Sha512};
 
 const VERSION_LABEL: &[u8] = b"HPKE-v1";
@@ -20,6 +18,11 @@ pub trait Kdf {
 
     /// The algorithm identifier for a KDF implementation
     const KDF_ID: u16;
+
+    /// Returns the size (in bytes) of the output of this KDF's Extract() function
+    fn extracted_key_size() -> usize {
+        <Self::HashImpl as FixedOutput>::OutputSize::to_usize()
+    }
 }
 
 // We use Kdf as a type parameter, so this is to avoid ambiguity.
@@ -67,14 +70,14 @@ impl KdfTrait for HkdfSha512 {
 
 /// Uses the given IKM to extract a secret, and then uses that secret, plus the given suite ID and
 /// info string, to expand to the output buffer
-pub(crate) fn extract_and_expand<Kem: KemTrait>(
+pub(crate) fn extract_and_expand<Kdf: KdfTrait>(
     ikm: &[u8],
     suite_id: &[u8],
     info: &[u8],
     out: &mut [u8],
 ) -> Result<(), hkdf::InvalidLength> {
     // Extract using given IKM
-    let (_, hkdf_ctx) = labeled_extract::<Kem::Kdf>(&[], suite_id, b"eae_prk", ikm);
+    let (_, hkdf_ctx) = labeled_extract::<Kdf>(&[], suite_id, b"eae_prk", ikm);
     // Expand using given info string
     hkdf_ctx.labeled_expand(suite_id, b"shared_secret", info, out)
 }
