@@ -14,7 +14,7 @@
 //! #     aead::ChaCha20Poly1305,
 //! #     kdf::HkdfSha384,
 //! #     kem::X25519HkdfSha256,
-//! #     EncappedKey, Kem as KemTrait, OpModeR, OpModeS, setup_receiver, setup_sender,
+//! #     Kem as KemTrait, OpModeR, OpModeS, setup_receiver, setup_sender,
 //! # };
 //! // These types define the ciphersuite Alice and Bob will be using
 //! type Kem = X25519HkdfSha256;
@@ -36,16 +36,14 @@
 //!     hpke::setup_sender::<Aead, Kdf, Kem, _>(&OpModeS::Base, &bob_pk, info_str, &mut csprng)
 //!         .expect("invalid server pubkey!");
 //!
-//! // Alice encrypts a message to Bob. msg gets encrypted in place, and aad is authenticated
-//! // associated data that is not encrypted.
-//! let mut msg = *b"fronthand or backhand?";
+//! // Alice encrypts a message to Bob. `aad` is authenticated associated data that is not
+//! // encrypted.
+//! let msg = b"fronthand or backhand?";
 //! let aad = b"a gentleman's game";
-//! let auth_tag = encryption_context
-//!     .seal(&mut msg, aad)
-//!     .expect("encryption failed!");
-//! // The msg was encrypted in-place. So rename it for clarity
-//! let ciphertext = msg;
-//! # let mut ciphertext = ciphertext; // Make it mutable so Bob can decrypt in place
+//! // To seal without allocating:
+//! //     let auth_tag = encryption_context.seal_in_place_detached(&mut msg, aad)?;
+//! // To seal with allocating:
+//! let ciphertext = encryption_context.seal(msg, aad).expect("encryption failed!");
 //!
 //! // ~~~
 //! // Alice sends the encapsulated key, message ciphertext, AAD, and auth tag to Bob over the
@@ -61,9 +59,10 @@
 //!         &encapsulated_key,
 //!         info_str,
 //!     ).expect("failed to set up receiver!");
-//! decryption_context.open(&mut ciphertext, aad, &auth_tag).expect("invalid ciphertext!");
-//! // The ciphertext was decrypted in-place. So rename it for clarity
-//! let plaintext = ciphertext;
+//! // To open without allocating:
+//! //     decryption_context.open_in_place_detached(&mut ciphertext, aad, &auth_tag)
+//! // To open with allocating:
+//! let plaintext = decryption_context.open(&ciphertext, aad).expect("invalid ciphertext!");
 //!
 //! assert_eq!(&plaintext, b"fronthand or backhand?");
 //! # }
@@ -77,11 +76,17 @@
 #[macro_use]
 extern crate std;
 
+#[cfg(feature = "std")]
+pub(crate) use std::vec::Vec;
+
 #[cfg(not(feature = "std"))]
 #[allow(unused_imports)]
 #[macro_use]
 extern crate alloc;
 
+#[cfg(not(feature = "std"))]
+pub(crate) use alloc::vec::Vec;
+
 //-------- Testing stuff --------//
 
 // kat_tests tests all the implemented ciphersuites, and thus needs all the dependencies. It also
@@ -99,38 +104,41 @@ mod test_util;
 
 //-------- Modules and exports--------//
 
-// Re-export this version of generic_array, since that's what's used everywhere in this crate
+// Re-export our versions of generic_array and rand_core, since their traits and types are exposed
+// in this crate
 pub use generic_array;
+pub use rand_core;
 
 #[macro_use]
 mod util;
 
 pub mod aead;
+mod dhkex;
 pub mod kdf;
 pub mod kem;
-pub mod kex;
-pub mod op_mode;
-pub mod setup;
-pub mod single_shot;
+mod op_mode;
+mod setup;
+mod single_shot;
 
 #[cfg(feature = "serde_impls")]
 mod serde_impls;
 
 #[doc(inline)]
-pub use crate::aead::{AeadCtxR, AeadCtxS};
-#[doc(inline)]
-pub use kem::{EncappedKey, Kem};
-#[doc(inline)]
-pub use kex::{Deserializable, Serializable};
+pub use kem::Kem;
 #[doc(inline)]
 pub use op_mode::{OpModeR, OpModeS, PskBundle};
 #[doc(inline)]
 pub use setup::{setup_receiver, setup_sender};
 #[doc(inline)]
-pub use single_shot::{single_shot_open, single_shot_seal};
+pub use single_shot::{
+    single_shot_open, single_shot_open_in_place_detached, single_shot_seal,
+    single_shot_seal_in_place_detached,
+};
 
 //-------- Top-level types --------//
 
+use generic_array::{typenum::marker_traits::Unsigned, ArrayLength, GenericArray};
+
 /// Describes things that can go wrong in the HPKE protocol
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum HpkeError {
@@ -172,6 +180,23 @@ impl core::fmt::Display for HpkeError {
     }
 }
 
+/// Implemented by types that have a fixed-length byte representation
+pub trait Serializable {
+    type OutputSize: ArrayLength<u8>;
+
+    fn to_bytes(&self) -> GenericArray<u8, Self::OutputSize>;
+
+    /// Returns the size (in bytes) of this type when serialized
+    fn size() -> usize {
+        Self::OutputSize::to_usize()
+    }
+}
+
+/// Implemented by types that can be deserialized from byte representation
+pub trait Deserializable: Serializable + Sized {
+    fn from_bytes(encoded: &[u8]) -> Result<Self, HpkeError>;
+}
+
 // An Error type is just something that's Debug and Display
 #[cfg(feature = "std")]
 impl std::error::Error for HpkeError {}
