@@ -1,11 +1,10 @@
 use crate::{
     aead::{Aead, AeadTag},
     kdf::Kdf as KdfTrait,
-    kem::{EncappedKey, Kem as KemTrait},
-    kex::KeyExchange,
+    kem::Kem as KemTrait,
     op_mode::{OpModeR, OpModeS},
     setup::{setup_receiver, setup_sender},
-    HpkeError,
+    HpkeError, Vec,
 };
 
 use rand_core::{CryptoRng, RngCore};
@@ -16,23 +15,23 @@ use rand_core::{CryptoRng, RngCore};
 //   ct = ctx.Seal(aad, pt)
 //   return enc, ct
 
-/// Does a `setup_sender` and `AeadCtx::seal` in one shot. That is, it does a key encapsulation to
-/// the specified recipient and encrypts the provided plaintext in place. See `setup::setup_sender`
-/// and `AeadCtx::seal` for more detail.
+/// Does a `setup_sender` and `AeadCtxS::seal_in_place_detached` in one shot. That is, it does a
+/// key encapsulation to the specified recipient and encrypts the provided plaintext in place. See
+/// `setup::setup_sender` and `AeadCtxS::seal_in_place_detached` for more detail.
 ///
 /// Return Value
 /// ============
 /// Returns `Ok((encapped_key, auth_tag))` on success. If an error happened during key
 /// encapsulation, returns `Err(HpkeError::EncapError)`. If an error happened during encryption,
 /// returns `Err(HpkeError::SealError)`. In this case, the contents of `plaintext` is undefined.
-pub fn single_shot_seal<A, Kdf, Kem, R>(
-    mode: &OpModeS<Kem::Kex>,
-    pk_recip: &<Kem::Kex as KeyExchange>::PublicKey,
+pub fn single_shot_seal_in_place_detached<A, Kdf, Kem, R>(
+    mode: &OpModeS<Kem>,
+    pk_recip: &Kem::PublicKey,
     info: &[u8],
     plaintext: &mut [u8],
     aad: &[u8],
     csprng: &mut R,
-) -> Result<(EncappedKey<Kem::Kex>, AeadTag<A>), HpkeError>
+) -> Result<(Kem::EncappedKey, AeadTag<A>), HpkeError>
 where
     A: Aead,
     Kdf: KdfTrait,
@@ -43,29 +42,61 @@ where
     let (encapped_key, mut aead_ctx) =
         setup_sender::<A, Kdf, Kem, R>(mode, pk_recip, info, csprng)?;
     // Encrypt
-    let tag = aead_ctx.seal(plaintext, aad)?;
+    let tag = aead_ctx.seal_in_place_detached(plaintext, aad)?;
 
     Ok((encapped_key, tag))
 }
 
+/// Does a `setup_sender` and `AeadCtxS::seal` in one shot. That is, it does a key encapsulation to
+/// the specified recipient and encrypts the provided plaintext. See `setup::setup_sender` and
+/// `AeadCtxS::seal` for more detail}
+///
+/// Return Value
+/// ============
+/// Returns `Ok((encapped_key, ciphertext))` on success. If an error happened during key
+/// encapsulation, returns `Err(HpkeError::EncapError)`. If an error happened during encryption,
+/// returns `Err(HpkeError::SealError)`.
+pub fn single_shot_seal<A, Kdf, Kem, R>(
+    mode: &OpModeS<Kem>,
+    pk_recip: &Kem::PublicKey,
+    info: &[u8],
+    plaintext: &[u8],
+    aad: &[u8],
+    csprng: &mut R,
+) -> Result<(Kem::EncappedKey, Vec<u8>), HpkeError>
+where
+    A: Aead,
+    Kdf: KdfTrait,
+    Kem: KemTrait,
+    R: CryptoRng + RngCore,
+{
+    // Encap a key
+    let (encapped_key, mut aead_ctx) =
+        setup_sender::<A, Kdf, Kem, R>(mode, pk_recip, info, csprng)?;
+    // Encrypt
+    let ciphertext = aead_ctx.seal(plaintext, aad)?;
+
+    Ok((encapped_key, ciphertext))
+}
+
 // draft11 §6.1
 // def OpenAuthPSK(enc, skR, info, aad, ct, psk, psk_id, pkS):
 //   ctx = SetupAuthPSKR(enc, skR, info, psk, psk_id, pkS)
 //   return ctx.Open(aad, ct)
 
-/// Does a `setup_receiver` and `AeadCtx::open` in one shot. That is, it does a key decapsulation
-/// for the specified recipient and decrypts the provided ciphertext in-place. See
-/// `setup::setup_reciever` and `AeadCtx::open` for more detail.
+/// Does a `setup_receiver` and `AeadCtxR::open_in_place_detached` in one shot. That is, it does a
+/// key decapsulation for the specified recipient and decrypts the provided ciphertext in place.
+/// See `setup::setup_reciever` and `AeadCtxR::open_in_place_detached` for more detail.
 ///
 /// Return Value
 /// ============
 /// Returns `Ok()` on success. If an error happened during key decapsulation, returns
 /// `Err(HpkeError::DecapError)`. If an error happened during decryption, returns
 /// `Err(HpkeError::OpenError)`. In this case, the contents of `ciphertext` is undefined.
-pub fn single_shot_open<A, Kdf, Kem>(
-    mode: &OpModeR<Kem::Kex>,
-    sk_recip: &<Kem::Kex as KeyExchange>::PrivateKey,
-    encapped_key: &EncappedKey<Kem::Kex>,
+pub fn single_shot_open_in_place_detached<A, Kdf, Kem>(
+    mode: &OpModeR<Kem>,
+    sk_recip: &Kem::PrivateKey,
+    encapped_key: &Kem::EncappedKey,
     info: &[u8],
     ciphertext: &mut [u8],
     aad: &[u8],
@@ -79,7 +110,35 @@ where
     // Decap the key
     let mut aead_ctx = setup_receiver::<A, Kdf, Kem>(mode, sk_recip, encapped_key, info)?;
     // Decrypt
-    aead_ctx.open(ciphertext, aad, tag)
+    aead_ctx.open_in_place_detached(ciphertext, aad, tag)
+}
+
+/// Does a `setup_receiver` and `AeadCtxR::open` in one shot. That is, it does a key decapsulation
+/// for the specified recipient and decrypts the provided ciphertext. See `setup::setup_reciever`
+/// and `AeadCtxR::open` for more detail.
+///
+/// Return Value
+/// ============
+/// Returns `Ok(plaintext)` on success. If an error happened during key decapsulation, returns
+/// `Err(HpkeError::DecapError)`. If an error happened during decryption, returns
+/// `Err(HpkeError::OpenError)`.
+pub fn single_shot_open<A, Kdf, Kem>(
+    mode: &OpModeR<Kem>,
+    sk_recip: &Kem::PrivateKey,
+    encapped_key: &Kem::EncappedKey,
+    info: &[u8],
+    ciphertext: &[u8],
+    aad: &[u8],
+) -> Result<Vec<u8>, HpkeError>
+where
+    A: Aead,
+    Kdf: KdfTrait,
+    Kem: KemTrait,
+{
+    // Decap the key
+    let mut aead_ctx = setup_receiver::<A, Kdf, Kem>(mode, sk_recip, encapped_key, info)?;
+    // Decrypt
+    aead_ctx.open(ciphertext, aad)
 }
 
 #[cfg(test)]
@@ -90,7 +149,7 @@ mod test {
         kdf::HkdfSha256,
         kem::Kem as KemTrait,
         op_mode::{OpModeR, OpModeS, PskBundle},
-        test_util::{gen_rand_buf, kex_gen_keypair},
+        test_util::gen_rand_buf,
     };
 
     use rand::{rngs::StdRng, SeedableRng};
@@ -105,7 +164,6 @@ mod test {
                 type A = $aead;
                 type Kdf = $kdf;
                 type Kem = $kem;
-                type Kex = <Kem as KemTrait>::Kex;
 
                 let msg = b"Good night, a-ding ding ding ding ding";
                 let aad = b"Five four three two one";
@@ -121,25 +179,24 @@ mod test {
                 };
 
                 // Generate the sender's and receiver's long-term keypairs
-                let (sk_sender_id, pk_sender_id) = kex_gen_keypair::<Kex, _>(&mut csprng);
-                let (sk_recip, pk_recip) = kex_gen_keypair::<Kex, _>(&mut csprng);
+                let (sk_sender_id, pk_sender_id) = Kem::gen_keypair(&mut csprng);
+                let (sk_recip, pk_recip) = Kem::gen_keypair(&mut csprng);
 
                 // Construct the sender's encryption context, and get an encapped key
-                let sender_mode = OpModeS::<Kex>::AuthPsk(
+                let sender_mode = OpModeS::<Kem>::AuthPsk(
                     (sk_sender_id, pk_sender_id.clone()),
                     psk_bundle.clone(),
                 );
 
                 // Use the encapped key to derive the reciever's encryption context
-                let receiver_mode = OpModeR::<Kex>::AuthPsk(pk_sender_id, psk_bundle);
+                let receiver_mode = OpModeR::<Kem>::AuthPsk(pk_sender_id, psk_bundle);
 
                 // Encrypt with the first context
-                let mut ciphertext = msg.clone();
-                let (encapped_key, tag) = single_shot_seal::<A, Kdf, Kem, _>(
+                let (encapped_key, ciphertext) = single_shot_seal::<A, Kdf, Kem, _>(
                     &sender_mode,
                     &pk_recip,
-                    &info[..],
-                    &mut ciphertext[..],
+                    info,
+                    msg,
                     aad,
                     &mut csprng,
                 )
@@ -149,19 +206,16 @@ mod test {
                 assert!(&ciphertext[..] != &msg[..]);
 
                 // Decrypt with the second context
-                single_shot_open::<A, Kdf, Kem>(
+                let decrypted = single_shot_open::<A, Kdf, Kem>(
                     &receiver_mode,
                     &sk_recip,
                     &encapped_key,
                     info,
-                    &mut ciphertext[..],
+                    &ciphertext,
                     aad,
-                    &tag,
                 )
                 .expect("single_shot_open() failed");
-                // Change name for clarity
-                let decrypted = ciphertext;
-                assert_eq!(&decrypted[..], &msg[..]);
+                assert_eq!(&decrypted, &msg);
             }
         };
     }
